CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
98 lines
4.1 KiB
Plaintext
98 lines
4.1 KiB
Plaintext
// Copyright (c) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
|
|
//
|
|
// Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
// of this software and associated documentation files (the "Software"), to deal
|
|
// in the Software without restriction, including without limitation the rights
|
|
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
// copies of the Software, and to permit persons to whom the Software is
|
|
// furnished to do so, subject to the following conditions:
|
|
//
|
|
// The above copyright notice and this permission notice shall be included in all
|
|
// copies or substantial portions of the Software.
|
|
//
|
|
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
|
|
// SOFTWARE.
|
|
|
|
#include <cuda_runtime.h>
|
|
|
|
#include <thrust/device_vector.h>
|
|
#include <thrust/execution_policy.h> // For thrust::device
|
|
#include <thrust/reduce.h>
|
|
#include <thrust/sequence.h>
|
|
|
|
#include <iostream>
|
|
|
|
// This example shows how to execute a Thrust device algorithm on an explicit
|
|
// CUDA stream. The simple program below fills a vector with the numbers
|
|
// [0, 1000) (thrust::sequence) and then performs a scan operation
|
|
// (thrust::inclusive_scan) on them. Both algorithms are executed on the same
|
|
// custom CUDA stream using the CUDA execution policies.
|
|
//
|
|
// Thrust provides two execution policies that accept CUDA streams that differ
|
|
// in when/if they synchronize the stream:
|
|
// 1. thrust::cuda::par.on(stream)
|
|
// - `stream` will *always* be synchronized before an algorithm returns.
|
|
// - This is the default `thrust::device` policy when compiling with the
|
|
// CUDA device backend.
|
|
// 2. thrust::cuda::par_nosync.on(stream)
|
|
// - `stream` will only be synchronized when necessary for correctness
|
|
// (e.g., returning a result from `thrust::reduce`). This is a hint that
|
|
// may be ignored by an algorithm's implementation.
|
|
|
|
int main()
|
|
{
|
|
thrust::device_vector<int> d_vec(1000);
|
|
|
|
// Create the stream:
|
|
cudaStream_t custom_stream;
|
|
cudaError_t err = cudaStreamCreate(&custom_stream);
|
|
if (err != cudaSuccess)
|
|
{
|
|
std::cerr << "Error creating stream: " << cudaGetErrorString(err) << "\n";
|
|
return 1;
|
|
}
|
|
|
|
// Construct a new `nosync` execution policy with the custom stream
|
|
auto nosync_exec_policy = thrust::cuda::par_nosync.on(custom_stream);
|
|
|
|
// Fill the vector with sequential data.
|
|
// This will execute using the custom stream and the stream will *not* be
|
|
// synchronized before the function returns, meaning asynchronous work may
|
|
// still be executing after returning and the contents of `d_vec` are
|
|
// undefined. Synchronization is not needed here because the following
|
|
// `inclusive_scan` is executed on the same stream and is therefore guaranteed
|
|
// to be ordered after the `sequence`
|
|
thrust::sequence(nosync_exec_policy, d_vec.begin(), d_vec.end());
|
|
|
|
// Construct a new *synchronous* execution policy with the same custom stream
|
|
auto sync_exec_policy = thrust::cuda::par.on(custom_stream);
|
|
|
|
// Compute in-place inclusive sum scan of data in the vector.
|
|
// This also executes in the custom stream, but the execution policy ensures
|
|
// the stream is synchronized before the algorithm returns. This guarantees
|
|
// there is no pending asynchronous work and the contents of `d_vec` are
|
|
// immediately accessible.
|
|
thrust::inclusive_scan(sync_exec_policy, d_vec.cbegin(), d_vec.cend(), d_vec.begin());
|
|
|
|
// This access is only valid because the stream has been synchronized
|
|
int sum = d_vec.back();
|
|
|
|
// Free the stream:
|
|
err = cudaStreamDestroy(custom_stream);
|
|
if (err != cudaSuccess)
|
|
{
|
|
std::cerr << "Error destroying stream: " << cudaGetErrorString(err) << "\n";
|
|
return 1;
|
|
}
|
|
|
|
// Print the sum:
|
|
std::cout << "sum is " << sum << std::endl;
|
|
|
|
return 0;
|
|
}
|